import sys
import signal
import typing
import asyncio
import logging
from functools import partial

import yaml
from yaml.scanner import ScannerError
import aiolifx

from eelifx.util import wait_for_members, marshal_commanders, shutdown_loop, normalise_endpoint
from eelifx.bulbs import Bulbs

from .processor import queryship

DEFAULT_CONFIG = {
    'poll_interval': 5,
    'wait_for_members': True,
    'groups': [
        {
            'match': '.*',
            'max_luminance': 0.29,
            'colour_temp': 3500,
            'base_state': '''
lifx_commanders[lc_index].set_power(True)
lifx_commanders[lc_index].set_colour("white")
lifx_commanders[lc_index].set_luminance(1.0)
''',
            'rules': [
                {
                    'statement': 'ship.energy > 0.6',
                    'effect': '''
lifx_commanders[lc_index].set_power(True)
lifx_commanders[lc_index].set_luminance(1.0)
''',
                },
                {
                    'statement': 'ship.energy < 0.4 and ship.energy >= 0.15',
                    'effect': '''
lifx_commanders[lc_index].set_power(True)
lifx_commanders[lc_index].set_luminance(0.6)
''',
                },
                {
                    'statement': 'ship.energy < 0.15 and ship.energy >= 0.05',
                    'effect': '''
lifx_commanders[lc_index].set_power(True)
lifx_commanders[lc_index].set_luminance(0.1)
''',
                },
                {
                    'statement': 'ship.energy < 0.05',
                    'effect': 'lifx_commanders[lc_index].set_power(False)',
                },
                {
                    'statement': "ship.alert_level == 'normal'",
                    'effect': "lifx_commanders[lc_index].set_colour('white')",
                },
                {
                    'statement': "ship.alert_level == 'YELLOW ALERT'",
                    'effect': '''
lifx_commanders[lc_index].set_colour('yellow')
lifx_commanders[lc_index].set_waveform({'waveform': 'SAW', 'hz': 0.5, 'alt_colour': 'white' })
''',
                },
                {
                    'statement': "ship.alert_level == 'RED ALERT'",
                    'effect': '''
lifx_commanders[lc_index].set_colour('red')
lifx_commanders[lc_index].set_waveform({'waveform': 'SAW', 'hz': 0.5, 'alt_colour': 'white' })
''',
                },
                {
                    'statement': 'ship.hull < 0.2 and ship.hull > 0.1',
                    'effect': '''
lifx_commanders[lc_index].set_power(True)
lifx_commanders[lc_index].set_waveform({'waveform': 'SAW', 'hz':5, 'alt_colour':'black', 'duty_cycle':0.9})
''',
                },
                {
                    'statement': 'ship.hull < 0.1',
                    'effect': 'lifx_commanders[lc_index].set_power(False)',
                },
            ]
        },
    ],
}

UDP_BROADCAST_PORT = 56700


def setup_logging(level=logging.INFO):
    logger = logging.getLogger()
    logger.setLevel(level)
    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(level)
    # create formatter
    formatter = logging.Formatter('%(asctime)s:%(name)s:[%(levelname)s]: %(message)s')
    # add formatter to ch
    ch.setFormatter(formatter)
    # add ch to logger
    logger.addHandler(ch)


def dump_config():
    return yaml.dump(DEFAULT_CONFIG, default_flow_style=False)


def display_config():
    print(dump_config())


def load_config(f_handle: typing.io):
    config = None

    try:
        config = yaml.load(f_handle.read())
    except ScannerError as e:
        logging.error(f'Unable to load config: {e}')
    finally:
        f_handle.close()

    return config


def setup_loop(
    mode: str,
    config=None,
    endpoint=None,
    loglevel=logging.DEBUG
):
    setup_logging(loglevel)

    if config is None:
        config = DEFAULT_CONFIG
    else:
        config = load_config(config)

    if config is None:
        sys.exit(1)

    if mode == 'run' and endpoint is None:
        logging.error('Must supply an endpoint!')
        sys.exit(1)

    if endpoint:
        config['endpoint'] = normalise_endpoint(endpoint)

    MyBulbs = Bulbs()

    lifx_commanders = marshal_commanders(config)
    #  Initialise the event loop
    loop = asyncio.get_event_loop()
    coro = loop.create_datagram_endpoint(
        partial(aiolifx.LifxDiscovery, loop, MyBulbs),
        local_addr=('0.0.0.0', UDP_BROADCAST_PORT)
    )

    try:
        loop.create_task(coro)
        asyncio.ensure_future(
            wait_for_members(
                loop,
                MyBulbs,
                lifx_commanders,
                config['poll_interval'],
                config,
                mode=mode
            )
        )
        logging.info("Use Ctrl-C to quit")
        loop.add_signal_handler(signal.SIGHUP, partial(shutdown_loop, loop))
        loop.add_signal_handler(signal.SIGTERM, partial(shutdown_loop, loop))
        loop.run_forever()
    except (SystemExit, KeyboardInterrupt):
        pass
    except Exception:
        logging.error()
    finally:
        shutdown_loop(loop)
        loop.stop()
        loop.run_forever()
        loop.close()


def setup_queryship(endpoint: str, hull: int=None, energy: int=None, loglevel=logging.DEBUG):
    setup_logging()
    loop = asyncio.get_event_loop()
    loop.add_signal_handler(signal.SIGHUP, partial(shutdown_loop, loop))
    loop.add_signal_handler(signal.SIGTERM, partial(shutdown_loop, loop))
    loop.run_until_complete(queryship(normalise_endpoint(endpoint), hull=hull, energy=energy))



import typing

from aiolifx import aiolifx


class Bulbs():
    """ A simple class with a register and  unregister methods
    """
    def __init__(self):
        self.bulbs = []

    def register(self, bulb):
        bulb.get_label()
        bulb.get_location()
        bulb.get_version()
        bulb.get_group()
        bulb.get_wififirmware()
        bulb.get_hostfirmware()
        self.bulbs.append(bulb)
        self.bulbs.sort(key=lambda x: x.label or x.mac_addr)

    def unregister(self, bulb):
        idx = 0
        for x in list([y.mac_addr for y in self.bulbs]):
            if x == bulb.mac_addr:
                del(self.bulbs[idx])
                break
            idx += 1

    def filter_group(self, compiled_regex: typing.re) -> typing.Generator[aiolifx.Light, None, None]:

        for bulb in self.bulbs:
            if compiled_regex.match(bulb.get_group()):
                yield bulb



import json
import typing
import asyncio
import logging

import aiohttp
from aiohttp.client_exceptions import ClientConnectorError

from eelifx.bulbs import Bulbs
from eelifx.luacode import luacode, statement_for
from eelifx.ship import Ship
from eelifx.lifx_commander import LifxCommander
from eelifx.transport import get_ship_status


def run_rules(
    lc_index: int,
    lifx_commanders: typing.List[LifxCommander],
    ship: Ship,
    rules: typing.Dict
):
    for rule in rules:
        logging.debug(rule)

        if eval(rule['statement_compiled']):
            exec(rule['effect_compiled'])


async def process_game_state(
    loop: asyncio.AbstractEventLoop,
    bulbs: Bulbs,
    lifx_commanders: typing.Sequence[LifxCommander],
    poll_interval: int,
    groups: typing.Dict,
    endpoint: str,
    ship: Ship=None,
):
    if ship is None:
        logging.info('Creating Ship object')
        ship = Ship()

    session = aiohttp.ClientSession()

    html = None
    try:
        html = await get_ship_status(session, endpoint)
    except ClientConnectorError as e:
        logging.warn("Unable to connect to EmptyEpsilon, is its http server running?")
        logging.warn(e)

    ship_data = None

    if html:
        try:
            ship_data = json.loads(html)
        except json.JSONDecodeError:
            logging.warning('Unable to parse EmptyEpsilon response')
        else:
            if 'ERROR' in ship_data.keys():
                if ship_data['ERROR'] == 'No game':
                    logging.info('EmptyEpsilon reports no game is running')
                else:
                    logging.critical('Error returned by EmptyEpsilon:%s' % ship_data['ERROR'])
                    logging.critical('Executed LUA:\n---%s---' % luacode())

                ship_data = None

    if ship_data:
        try:
            ship.update(ship_data)
        except Exception as e:
            raise
        else:
            logging.info(ship)

            for lc_index, group in enumerate(groups):
                lifx_commanders[lc_index].reset()
                exec(group['base_state_compiled'])

                run_rules(
                    lc_index,
                    lifx_commanders,
                    ship,
                    group['rules']
                )

                lifx_commanders[lc_index].apply(bulbs)

    session.close()
    await asyncio.sleep(poll_interval)
    asyncio.ensure_future(process_game_state(loop, bulbs, lifx_commanders, poll_interval, groups, endpoint, ship))


async def queryship(endpoint, hull=None, energy=None):
    session = aiohttp.ClientSession()
    insert = ''

    if hull:
        insert = f"{insert}{statement_for('hull', hull)}"

    if energy:
        insert = f"{insert}{statement_for('energy', energy)}"

    html = None
    try:
        html = await get_ship_status(session, endpoint, insert=insert)
    except ClientConnectorError as e:
        logging.warn("Unable to connect to EmptyEpsilon, is its http server running?")
        logging.warn(e)

    logging.info(f'Response\n{html}')
    session.close()



import sys
import typing
import asyncio
import logging
from copy import deepcopy
from pprint import pformat

from eelifx.bulbs import Bulbs
from eelifx.lifx_commander import LifxCommander
from eelifx.processor import process_game_state


def shutdown_loop(loop):
    logging.info('Cancelling tasks...')
    for task in asyncio.Task.all_tasks():
        task.cancel()
    logging.info('Bye, exiting soon...')


def compile_items(uncomplied: typing.List[typing.Dict], item_key: str, mode: str='exec'):
    res = deepcopy(uncomplied)
    for item in res:
        if item_key not in item:
            logging.warning(
                "Could not find %s in object %s.",
                item_key,
                item
            )
            next
        try:
            item['%s_compiled' % item_key] = compile(item[item_key], '<string>', mode)
        except Exception as e:
            logging.exception("Unable to compile the following code to a python code object: %s" % pformat(item[item_key]))
            raise
    return res


async def wait_for_members(
    loop: asyncio.AbstractEventLoop,
    bulbs: Bulbs,
    lifx_commanders: typing.Sequence[LifxCommander],
    poll_interval: int,
    config: typing.Dict,
    mode: str
):
    ok = True

    if mode in ['grouptest', 'reset']:
        logging.info(f'Forcing wait_for_members on for {mode}')
        config['wait_for_members'] = True

    if 'wait_for_members' in config and config['wait_for_members']:
        logging.info('Checking to see if our groups have members yet')
        if not all([l.has_members(bulbs) for l in lifx_commanders]):
            ok = False
            logging.info(
                'At least one group has no members and wait_for_members is set to true, will retry in %s seconds.',
                poll_interval
            )
    # requeue check
    if not ok:
        await asyncio.sleep(poll_interval)
        asyncio.ensure_future(wait_for_members(loop, bulbs, lifx_commanders, poll_interval, config, mode))

    if ok:
        logging.info('At least one globe in each group is present, or wait_for_members is set to false.')
        logging.info('Compiling base states...')
        config['groups'] = compile_items(config['groups'], 'base_state', mode='exec')

        groups = []

        for group in config['groups']:
            groups.append(deepcopy(group))
            logging.info('Compiling rule statements...')
            groups[-1]['rules'] = compile_items(groups[-1]['rules'], 'statement', mode='eval')
            logging.info('Compiling effects statements...')
            groups[-1]['rules'] = compile_items(groups[-1]['rules'], 'effect', mode='exec')

        if mode == 'grouptest':
            asyncio.ensure_future(
                group_test(loop, bulbs, lifx_commanders, 5, groups, 0, 0)
            )
        elif mode == 'run':
            asyncio.ensure_future(
                process_game_state(loop, bulbs, lifx_commanders, poll_interval, groups, config['endpoint'])
            )
        else:
            asyncio.ensure_future(
                reset_lights(loop, bulbs, lifx_commanders, 5, groups)
            )


async def reset_lights(
    loop: asyncio.AbstractEventLoop,
    bulbs: Bulbs,
    lifx_commanders: typing.Sequence[LifxCommander],
    poll_interval: int,
    groups: typing.Dict
):
    for lc_index, group in enumerate(groups):
        lifx_commanders[lc_index].reset()
        exec(group['base_state_compiled'])
        lifx_commanders[lc_index].apply(bulbs)
    await asyncio.sleep(poll_interval)
    loop.stop()


async def group_test(
    loop: asyncio.AbstractEventLoop,
    bulbs: Bulbs,
    lifx_commanders: typing.Sequence[LifxCommander],
    poll_interval: int,
    groups: typing.Dict,
    group: int,
    rule: int
):
    logging.info(
        "Resetting lights to base state"
    )
    lc_index = group
    exec(groups[group]['base_state_compiled'])
    lifx_commanders[lc_index].apply(bulbs)
    await asyncio.sleep(poll_interval)
    logging.info(
        "[Group %s][Rule %s] Running block for condition '%s'",
        group,
        rule,
        groups[group]['rules'][rule]['statement']
    )
    exec(groups[group]['rules'][rule]['effect_compiled'])
    lifx_commanders[lc_index].apply(bulbs)
    lifx_commanders[lc_index].reset()
    await asyncio.sleep(poll_interval)
    rule += 1

    if rule == len(groups[group]['rules']):
        rule = 0
        group += 1

        if group == len(groups):
            logging.info('End of test cycle')
            loop.stop()

    asyncio.ensure_future(group_test(loop, bulbs, lifx_commanders, poll_interval, groups, group, rule))


def marshal_commanders(config):
    lifx_commanders = []
    for group in config['groups']:
        lifx_commanders.append(
            LifxCommander(
                poll_interval=config['poll_interval'],
                target_group=group['match'],
                max_luminance=group['max_luminance'],
                colour_temp=group['colour_temp']
            )
        )

    if len(lifx_commanders) == 0:
        logging.error('No groups have been defined in the config!')
        sys.exit(1)

    return lifx_commanders


def normalise_endpoint(endpoint):
    if not endpoint.startswith('http'):
        endpoint = f'http://{endpoint}'

    if not endpoint.endswith(':8080/exec.lua'):
        endpoint = f'{endpoint}:8080/exec.lua'

    if not endpoint.endswith('/exec.lua'):
        endpoint = f'{endpoint}/exec.lua'

    return endpoint



import re
import typing
import logging
from functools import wraps

from aiolifx import aiolifx
from colour import Color


def run_once(f):
    @wraps(f)
    def wrapper(*args, **kwds):
        if f.__name__ in args[0]._command_stack:
            return None

        return f(*args, **kwds)
    return wrapper


class LifxCommander():
    SUPPORTED_WAVEFORMS = {
        'SAW': {'code': 0, 'duty_cycle': 0.5},
        'SINE': {'code': 1, 'duty_cycle': 0.5},
        'HALF_SINE': {'code': 2, 'duty_cycle': 0.5},
        'TRIANGLE': {'code': 3, 'duty_cycle': 0.5},
        'PULSE': {'code': 4, 'duty_cycle': 0.5},
    }

    def __init__(self, poll_interval: int=20, max_luminance: float=1.0, target_group: str='.*', colour_temp: int=3500):
        self._poll_interval = poll_interval
        self._max_luminance = max_luminance
        self._command_stack = {}
        self._supported_effects = ['none', 'strobe', 'flicker']
        self._target_group = re.compile(target_group, flags=re.I)
        self._target_group_orig = target_group
        self._colour_temp = colour_temp

    def reset(self):
        self._command_stack = {}

    def set_colour(self, val):
        logging.debug('setting colour %s', val)
        self._command_stack['set_colour'] = val

    def set_waveform(self, val):
        logging.debug('setting waveform %s', val)

        if 'hz' not in val:
            raise ValueError('Must supply a rate (hz)')
        if 'waveform' not in val:
            raise ValueError('Must supply a waveform (%s)', LifxCommander.SUPPORTED_WAVEFORMS)
        if 'alt_colour' not in val:
            logging.info('Waveform alt_colour not supplied, it will be black')
            val['alt_colour'] = 'black'
        if 'duty_cycle' not in val:
            default_duty_cycle = LifxCommander.SUPPORTED_WAVEFORMS[val['waveform']]['duty_cycle']
            logging.info(f'Waveform duty_cycle not supplied, it will be {default_duty_cycle}')
            val['duty_cycle'] = default_duty_cycle

        if val['waveform'] not in LifxCommander.SUPPORTED_WAVEFORMS:
            raise ValueError('Call to set_waveform with unsupported waveform string')

        val['wave_code'] = LifxCommander.SUPPORTED_WAVEFORMS[val['waveform']]['code']

        self._command_stack['set_waveform'] = val

    def set_effect(self, val):
        assert val in self._supported_effects
        logging.debug('setting effect, %s', val)
        self._command_stack['set_effect'] = val

    def set_luminance(self, val):
        logging.debug('setting luminance %s', val)
        self._command_stack['set_luminance'] = val

    def set_power(self, val):
        '''
        Can be called as many times as required but latches off if called with false
        '''
        if 'set_power' in self._command_stack and not self._command_stack['set_power']:
            return

        logging.debug(f'setting power to {val}')
        self._command_stack['set_power'] = val

    def has_members(self, bulbs: typing.Sequence[aiolifx.Light]) -> bool:
        res = len(list(bulbs.filter_group(self._target_group))) > 0
        logging.info(
            'Group %s has %s',
            self._target_group_orig,
            'at least one member' if res else 'not got any members'
        )
        return res

    def _calculate_peroid_and_cycles(self, hz):
        peroid = 1000 / hz
        cycles = hz * self._poll_interval
        return peroid, cycles

    def _calculate_hsbk(self, _colour: Color):
        return [
            int(round((float(_colour.hue)*65535.0)/1.0)),
            int(round((float(_colour.saturation)*65535.0)/1.0)),
            int(round((float(_colour.luminance)*65535.0)/1.0)),
            self._colour_temp
        ]

    def apply(self, bulbs):
        if len(self._command_stack) == 0:
            logging.warning('Apply called but no commands registered')
            return

        logging.debug('Candidate bulbs are %s' % bulbs)
        command_stack = self._command_stack

        target_bulbs = bulbs.filter_group(self._target_group)

        m_colour = None

        for bulb in target_bulbs:
            if 'set_power' in command_stack:
                # change bulb's power if it differs
                desired_state = command_stack['set_power']
                observed_state = True if bulb.get_power() == 65535 else False

                if observed_state != desired_state:
                    bulb.set_power(desired_state)

                if not desired_state:
                    next

            if 'set_colour' in command_stack:
                m_colour = Color(command_stack['set_colour'])
                logging.debug(
                    'Commanded colour is %s (%s)',
                    m_colour,
                    m_colour.hsl
                )
                if m_colour.luminance > self._max_luminance:
                    logging.debug(
                        'Clipping colour\'s luminance to %s',
                        self._max_luminance
                    )
                    m_colour.luminance = self._max_luminance

            if 'set_luminance' in command_stack:
                if m_colour:
                    m_colour.luminance = m_colour.luminance * command_stack['set_luminance']
                else:
                    logging.info('Unable to scale luminance without colour being set')

            logging.debug(
                'Scaled colour is %s (%s)',
                m_colour,
                m_colour.hsl
            )

            set_colour_callback = None

            if 'set_waveform' in command_stack:
                data = command_stack['set_waveform']

                alt_colour = Color(data['alt_colour'])
                if alt_colour.luminance > self._max_luminance:
                    logging.debug(
                        'Clipping alt_colour\'s luminance to %s',
                        self._max_luminance
                    )
                    alt_colour.luminance = self._max_luminance

                if 'set_luminance' in command_stack:
                    alt_colour.luminance = alt_colour.luminance * command_stack['set_luminance']

                period, cycles = self._calculate_peroid_and_cycles(data['hz'])
                alt_colour_hsbk = self._calculate_hsbk(alt_colour)
                duty_cycle = data['duty_cycle']
                waveform = data['wave_code']

                def set_waveform_cb(light, response):
                    light.set_waveform(
                        {
                            'transient': False,
                            'color': alt_colour_hsbk,
                            'period': period,
                            'cycles': cycles,
                            'duty_cycle': duty_cycle,
                            'waveform': waveform
                        }
                    )

                set_colour_callback = set_waveform_cb

            bulb.set_color(self._calculate_hsbk(m_colour), callb=set_colour_callback)



import os
from setuptools import setup


def filter_requirements(fn):
    with open(fn) as fh:
        filtered_requirements = []
        for line in fh.readlines():
            if line[0] in ['#', ' ', '-']:
                continue
            filtered_requirements.append(line)
    return filtered_requirements


def load_version():
    with open('VERSION') as fh:
        res = fh.read()
    return res


version = load_version()

required = filter_requirements('requirements.txt')

required_test = filter_requirements('requirements-test.txt')

long_description = 'Install, then run ``eelifx --help``. See `the repo<https://github.com/cgspeck/eelifx>`_ for more information.'

if os.path.exists('README.rst'):
    with open('README.rst') as fh:
        long_description = fh.read()

setup(
    name='eelifx',
    description='Use game state in EmptyEpsilon to control your Lifx globes.',
    long_description=long_description,
    version=version,
    author='Chris Speck',
    author_email='cgspeck@gmail.com',
    url='https://github.com/cgspeck/eelifx',
    packages=['eelifx'],
    install_requires=required,
    extras_require={
        'tests': required_test
    },
    entry_points='''
        [console_scripts]
        eelifx=eelifx.cli:root
    ''',
    keywords=['EmptyEpsilon', 'Lifx'],
    download_url=f'https://github.com/cgspeck/eelifx/archive/{version}.tar.gz'
)



import async_timeout

from eelifx.luacode import luacode


async def fetch(session, url):
    with async_timeout.timeout(10):
        async with session.get(url) as response:
            return await response.text()


async def get_ship_status(session, url, insert=None):
    with async_timeout.timeout(10):
        async with session.post(url, data=luacode(insert)) as response:
            return await response.text()



def luacode(insert: str=None) -> str:

    if insert is None or len(insert) == 0:
        insert = ''
    else:
        insert = f'\n{insert}'

    return f'''
playerShip = getPlayerShip(-1){insert}
energyLevel = playerShip:getEnergyLevel()
energyLevelMax = playerShip:getEnergyLevelMax()
alertLevel = playerShip:getAlertLevel()
shieldsActive = playerShip:getShieldsActive()
hull = playerShip:getHull()
hullMax = playerShip:getHullMax()

return {{
    energyLevel=energyLevel,
    energyLevelMax=energyLevelMax,
    hull = hull,
    hullMax = hullMax,
    shieldsActive=shieldsActive,
    alertLevel=alertLevel,
}}
'''

def statement_for(action: str, value) -> str:
    if action == 'hull':
        return f'playerShip:setHull({value})\n'

    if action == 'energy':
        return f'playerShip:setEnergyLevel({value})\n'

    valid_actions = ['hull', 'energy']
    raise ValueError(f'Please supply action, being one of {valid_actions}')



import logging

import click

from eelifx.config import display_config, setup_loop, setup_queryship


def _call_loop(
    mode: str,
    endpoint: str=None,
    config=None,
    loglevel=None,
):
    if loglevel is None:
        loglevel = logging.INFO
    else:
        loglevel = getattr(logging, loglevel)

    setup_loop(
        mode,
        config=config,
        endpoint=endpoint,
        loglevel=loglevel
    )


@click.option(
    '--loglevel',
    default=None,
    help='E.g. "DEBUG" or "INFO"',
)
@click.group()
def root(loglevel=None):
    ''' EELifx Copyright (C) 2017 Chris Speck'''
    pass


@click.command(help='Print default config to standard out.')
def showconfig(loglevel=None):
    display_config()


@click.option(
    '--config',
    default=None,
    type=click.File('rt'),
    help='Path to optional config file with rules to load'
)
@click.command(
    short_help='Execute each rule in succession.',
    help='A test cycle which exercises each rule for each group in succession, resetting lights to base prior to each test.'
)
def grouptest(config=None, loglevel=None):
    _call_loop(
        'grouptest',
        endpoint=None,
        config=config,
        loglevel=loglevel
    )


@click.option(
    '--config',
    default=None,
    type=click.File('rt'),
    help='Path to optional config file with rules to load'
)
@click.argument(
    'endpoint'
)
@click.command(help='Poll and set lights according to game state.')
def run(endpoint, config=None, loglevel=None):
    _call_loop(
        'run',
        endpoint=endpoint,
        config=config,
        loglevel=loglevel
    )


@click.argument(
    'endpoint'
)
@click.option('hull', '-h', type=click.IntRange(0, 2000), help='Set hull level', metavar='<int>')
@click.option('energy', '-e', type=click.IntRange(0, 5000), help='Set energy level', metavar='<int>')
@click.command(short_help='Query EE and set parameters if given.')
def queryship(endpoint, loglevel=None, hull=None, energy=None):
    setup_queryship(endpoint, hull=hull, energy=energy, loglevel=loglevel)


@click.option(
    '--config',
    default=None,
    type=click.File('rt'),
    help='Path to optional config file with rules to load'
)
@click.command(help='Reset lights to base state.')
def reset(config=None, loglevel=None):
    _call_loop(
        'reset',
        endpoint=None,
        config=config,
        loglevel=loglevel
    )


root.add_command(showconfig)
root.add_command(grouptest)
root.add_command(run)
root.add_command(reset)
root.add_command(queryship)

if __name__ == '__main__':
    root()



class Ship():
    def __init__(self):
        self._alert_level = 'normal'
        self._shields_active = False
        self._shields_transitioned = False
        self._energy = 1.0
        self._hull = 1.0

    def update(self, lua_response):
        self._alert_level = lua_response['alertLevel']
        self._shields_transitioned = (self._shields_active != lua_response['shieldsActive'])
        self._shields_active = lua_response['shieldsActive']
        self._energy = lua_response['energyLevel'] / lua_response['energyLevelMax']
        self._hull = lua_response['hull'] / lua_response['hullMax']

    @property
    def alert_level(self):
        return self._alert_level

    @property
    def shields_active(self):
        return self._shields_active

    @property
    def shields_transitioned(self):
        return self._shields_transitioned

    @property
    def energy(self):
        return self._energy

    @property
    def hull(self):
        return self._hull

    def to_dict(self):
        data = {}
        data['alert_level'] = self.alert_level
        data['shields_active'] = self.shields_active
        data['shields_transitioned'] = self.shields_transitioned
        data['energy'] = self.energy
        data['hull'] = self.hull

        return data

    def __repr__(self):
        res = ''
        for k, v in self.to_dict().items():
            res += " {key}={value}".format(key=k, value=v)

        return '<Ship{res}>'.format(res=res)
